
# source helper functions used by underlying procedures.
source ttk_helper_s10tx.tcl

source readwrite.tcl

puts ""
puts ""
puts "Dynamic Reconfiguration options"
puts "-------------------------------"

###############################################################################
#reconfig_to_nrz
###############################################################################

proc reconfig_to_nrz {phy serial_loop kpfec} {
global high_datarate
#Reconfiguration based on dump
set high_datarate 0

puts "stream 25G NRZ ch 0"

source stream_25g_nrz_ch0.tcl

puts "stream 25G NRZ ch 1"

source stream_25g_nrz_ch1.tcl

puts "stream 25G NRZ ch 2"
source stream_25g_nrz_ch2.tcl

puts "stream 25G NRZ ch 3"
source stream_25g_nrz_ch3.tcl


after 1000

# Reset uC on all channels

reset_uc $phy 3 1
reset_uc $phy 2 1
reset_uc $phy 1 1
reset_uc $phy 0 1

after 1000

if {$kpfec == 0 } {
#Set to NRZ + tx_clk_divider and rx_clk_divider set to 165
set_mode_etile $phy 3 1 NRZ 0xA5 0xA5 0 0 1
set_mode_etile $phy 2 1 NRZ 0xA5 0xA5 0 0 1
set_mode_etile $phy 1 1 NRZ 0xA5 0xA5 0 0 1
set_mode_etile $phy 0 1 NRZ 0xA5 0xA5 0 0 1
} else {
#Set to NRZ + tx_clk_divider and rx_clk_divider set to 170
set_mode_etile $phy 3 1 NRZ 0xAA 0xAA 0 0 1
set_mode_etile $phy 2 1 NRZ 0xAA 0xAA 0 0 1
set_mode_etile $phy 1 1 NRZ 0xAA 0xAA 0 0 1
set_mode_etile $phy 0 1 NRZ 0xAA 0xAA 0 0 1
}

after 1000

#set ilb or elb

set_serial_loopback_phy_etile $phy $serial_loop 1

after 1000


#Assert IO reset 
set m [lindex [get_service_paths master] 1]  
open_service master $m

set base_addr 0x00000

s10_write_reg_32 $m $base_addr 0x310 0xffffffff 0x01


after 1000

#De-Assert IO reset 

s10_write_reg_32 $m $base_addr 0x310 0xffffffff 0x00


close_service master $m

puts "Reconfigure MAC and FEC to 4x25G Aggregate mode"
if {$kpfec == 0} {
	reconfig_fec_rsfec_aggregate 
	} else {
	reconfig_fec_kpfec_aggregate
}


after 1000

#Run iADP on the phy with 0.0 effort
iadp_phy $phy 0.0 1

}
puts {reconfig_to_nrz {phy serial_loop kpfec}}
puts {to reconfigure to CAUI-4 (4x25.78125 Gbps) NRZ with RSFEC(528,514) with serial loop    use reconfig_to_nrz etile_phy_1 1 0}
puts {to reconfigure to CAUI-4 (4x25.78125 Gbps) NRZ with RSFEC(528,514) without serial loop use reconfig_to_nrz etile_phy_1 0 0}
puts {to reconfigure to GAUI-4 (4x26.5625  Gbps) NRZ with RSFEC(544,514) with serial loop    use reconfig_to_nrz etile_phy_1 1 1}
puts {to reconfigure to GAUI-4 (4x26.5625  Gbps) NRZ with RSFEC(544,514) without serial loop use reconfig_to_nrz etile_phy_1 0 1}
puts ""


###############################################################################
#reconfig_to_pam4
###############################################################################

proc reconfig_to_pam4 {phy serial_loop} {
global high_datarate
#Reconfiguration based on dump

set high_datarate 0

puts "stream PAM4 ch 0"

source stream_pam4_ch0.tcl

puts "stream PAM4 ch 1"

source stream_pam4_ch1.tcl

puts "stream PAM4 ch 2"
source stream_pam4_ch2.tcl

puts "stream PAM4 ch 3"
source stream_pam4_ch3.tcl


after 1000


#Set high_datarate
set high_datarate 1

# Reset uC on all channels

reset_uc $phy 1 1
reset_uc $phy 0 1

after 1000

#Set to PAM4 + gray encoding + tx_clk_divider and rx_clk_divider set to 170.
set_mode_etile $phy 0 1 PAM4 0xAA 0xAA 1 0 1
set_mode_etile $phy 1 1 PAM4 0xAA 0xAA 1 0 1

after 1000

#set serial loopback on all lanes

set_serial_loopback_phy_etile $phy $serial_loop 1


after 1000


#Assert IO reset 
set m [lindex [get_service_paths master] 1]  
open_service master $m

set base_addr 0x00000

s10_write_reg_32 $m $base_addr 0x310 0xffffffff 0x01

s10_read_reg_32 $m $base_addr 0x40b

after 1000

#De-Assert IO reset 

s10_write_reg_32 $m $base_addr 0x310 0xffffffff 0x00


close_service master $m

puts "Reconfigure MAC and FEC to 2x50G Aggregate mode"
reconfig_fec_kpfec_aggregate


after 1000

#Run iADP on the phy with 0.5 effort (PAM4)
iadp_phy $phy 0.5 1
}
puts {reconfig_to_pam4 {phy serial_loop}}
puts {to reconfigure to CAUI-2 (2x53.125 Gbps) PAM4 with serial loop    use reconfig_to_pam4 etile_phy_1 1 }
puts {to reconfigure to CAUI-2 (2x53.125 Gbps) PAM4 without serial loop use reconfig_to_pam4 etile_phy_1 0 }
puts ""


###############################################################################
#reconfig_to_nrz_no_fec
###############################################################################
proc reconfig_to_nrz_no_fec {phy serial_loop} {
global high_datarate
#disable fec

set m [lindex [get_service_paths master] 1]  
open_service master $m

set base_addr 0x40000

#Register 0x04 : rsfec_top_clk_cfg
s10_write_reg $m $base_addr 0x4 0xff 0x00
s10_write_reg $m $base_addr 0x5 0xff 0x00
s10_write_reg $m $base_addr 0x6 0xff 0x00
s10_write_reg $m $base_addr 0x7 0xff 0x00

#Register 0x14 : rsfec_top_rx_cfg
s10_write_reg $m $base_addr 0x14 0xff 0x00
s10_write_reg $m $base_addr 0x15 0xff 0x00
s10_write_reg $m $base_addr 0x16 0xff 0x00
s10_write_reg $m $base_addr 0x17 0xff 0x00

close_service master $m


#Reconfiguration based on dump
set high_datarate 0

puts "stream 25G NRZ ch 0"

source stream_25g_nrz_ch0.tcl

#different xcvr reconfiguration when fec is disabled
rmw_channel $phy 0 0x004 0xFF 0xCB 0
rmw_channel $phy 0 0x005 0xFF 0x4C 0
rmw_channel $phy 0 0x006 0xFF 0x0F 0
rmw_channel $phy 0 0x007 0xFF 0xA6 0

puts "stream 25G NRZ ch 1"

source stream_25g_nrz_ch1.tcl

#different xcvr reconfiguration when fec is disabled
rmw_channel $phy 1 0x004 0xFF 0xCB 0
rmw_channel $phy 1 0x005 0xFF 0x4C 0
rmw_channel $phy 1 0x006 0xFF 0x0F 0
rmw_channel $phy 1 0x007 0xFF 0xA6 0


puts "stream 25G NRZ ch 2"
source stream_25g_nrz_ch2.tcl

#different xcvr reconfiguration when fec is disabled
rmw_channel $phy 2 0x004 0xFF 0xCB 0
rmw_channel $phy 2 0x005 0xFF 0x4C 0
rmw_channel $phy 2 0x006 0xFF 0x0F 0
rmw_channel $phy 2 0x007 0xFF 0xA6 0


puts "stream 25G NRZ ch 3"
source stream_25g_nrz_ch3.tcl

#different xcvr reconfiguration when fec is disabled
rmw_channel $phy 3 0x004 0xFF 0xCB 0
rmw_channel $phy 3 0x005 0xFF 0x4C 0
rmw_channel $phy 3 0x006 0xFF 0x0F 0
rmw_channel $phy 3 0x007 0xFF 0xA6 0


after 1000


# Reset uC on all channels

reset_uc $phy 3 1
reset_uc $phy 2 1
reset_uc $phy 1 1
reset_uc $phy 0 1

after 1000

#Set to NRZ + tx_clk_divider and rx_clk_divider set to 165
set_mode_etile $phy 3 1 NRZ 0xA5 0xA5 0 0 1
set_mode_etile $phy 2 1 NRZ 0xA5 0xA5 0 0 1
set_mode_etile $phy 1 1 NRZ 0xA5 0xA5 0 0 1
set_mode_etile $phy 0 1 NRZ 0xA5 0xA5 0 0 1

after 1000

#set serial loopback on all lanes

set_serial_loopback_phy_etile $phy $serial_loop 1


after 1000


#Assert IO reset 
set m [lindex [get_service_paths master] 1]  
open_service master $m

set base_addr 0x00000

s10_write_reg_32 $m $base_addr 0x310 0xffffffff 0x01


after 1000

#De-Assert IO reset 

s10_write_reg_32 $m $base_addr 0x310 0xffffffff 0x00


close_service master $m

puts "Reconfigure MAC with no FEC"
reconfig_no_fec


after 1000

#Run iADP on the phy with 0.0 effort
iadp_phy $phy 0.0 1
}
puts {reconfig_to_nrz_no_fec {phy serial_loop}}
puts {to reconfigure to CAUI-4 (4x25.78125 Gbps) NRZ without FEC with serial loop    use reconfig_to_nrz_no_fec etile_phy_1 1}
puts {to reconfigure to CAUI-4 (4x25.78125 Gbps) NRZ without FEC without serial loop use reconfig_to_nrz_no_fec etile_phy_1 0}
puts ""



###############################################################################
#reconfig_fec_rsfec_aggregate
###############################################################################

proc reconfig_fec_rsfec_aggregate {} {
set m [lindex [get_service_paths master] 1]  
open_service master $m


#Reconfigure MAC 

set base_addr 0x00000


#Timer Window for Hi-BER Checks Fields
s10_write_reg_32 $m $base_addr 0x37a 0xffffffff 0x312C7
#EHIP TX MAC Feature Configuration 
s10_write_reg_32 $m $base_addr 0x40b 0xffffffff 0x9FFD8028

#Setup MAC with FEC Enabled
#clear bit[5] of 0x313
s10_write_reg_32 $m $base_addr 0x313 0xffffffff 0x00000000
#clear bit[3] (not documented) and clear bit[9] (set use_aligner)
s10_write_reg_32 $m $base_addr 0x30E 0x208 0x00000000


#Reconfigure FEC to aggregate (4x25G)
#This is based on the readout.

#Fec base address *4

set base_addr 0x40000

#Register 0x04 : rsfec_top_clk_cfg
s10_write_reg $m $base_addr 0x4 0xff 0x00
s10_write_reg $m $base_addr 0x5 0xff 0x0F
s10_write_reg $m $base_addr 0x6 0xff 0x00
s10_write_reg $m $base_addr 0x7 0xff 0x00

#Register 0x10 : rsfec_top_tx_cfg
s10_write_reg $m $base_addr 0x10 0xff 0x00
s10_write_reg $m $base_addr 0x11 0xff 0x00
s10_write_reg $m $base_addr 0x12 0xff 0x00
s10_write_reg $m $base_addr 0x13 0xff 0x00

#Register 0x14 : rsfec_top_rx_cfg
s10_write_reg $m $base_addr 0x14 0xff 0x11
s10_write_reg $m $base_addr 0x15 0xff 0x11
s10_write_reg $m $base_addr 0x16 0xff 0x00
s10_write_reg $m $base_addr 0x17 0xff 0x00

#Register 0x20 : tx_aib_dsk_conf
s10_write_reg $m $base_addr 0x20 0xff 0x00
s10_write_reg $m $base_addr 0x21 0xff 0x00
s10_write_reg $m $base_addr 0x22 0xff 0x00
s10_write_reg $m $base_addr 0x23 0xff 0x00

#Register 0x30 : rsfec_core_cfg
s10_write_reg $m $base_addr 0x30 0xff 0x00
s10_write_reg $m $base_addr 0x31 0xff 0x00
s10_write_reg $m $base_addr 0x32 0xff 0x00
s10_write_reg $m $base_addr 0x33 0xff 0x00

#Register 0x40 : rsfec_lane_cfg
s10_write_reg $m $base_addr 0x40 0xff 0x00

#Register 0x44 : rsfec_lane_cfg
s10_write_reg $m $base_addr 0x44 0xff 0x00

#Register 0x48 : rsfec_lane_cfg
s10_write_reg $m $base_addr 0x48 0xff 0x00

#Register 0x4C : rsfec_lane_cfg
s10_write_reg $m $base_addr 0x4C 0xff 0x00


close_service master $m
}

###############################################################################
#reconfig_fec_kpfec_aggregate
###############################################################################

proc reconfig_fec_kpfec_aggregate {} {
set m [lindex [get_service_paths master] 1]  
open_service master $m



#Reconfigure MAC 

set base_addr 0x00000

#Timer Window for Hi-BER Checks Fields  
s10_write_reg_32 $m $base_addr 0x37a 0xffffffff 0x32a9e
#EHIP TX MAC Feature Configuration => Enable custom cadence
s10_write_reg_32 $m $base_addr 0x40b 0xffffffff 0x9FFD81e8

#Setup MAC with FEC Enabled
#clear bit[5] of 0x313
s10_write_reg_32 $m $base_addr 0x313 0xffffffff 0x00000000
#clear bit[3] (not documented) and clear bit[9] (set use_aligner)
s10_write_reg_32 $m $base_addr 0x30E 0x208 0x00000000


#Reconfigure FEC to aggregate (2x50G PAM4)
#This is based on the readout.

#Fec base address *4

set base_addr 0x40000

#Register 0x04 : rsfec_top_clk_cfg
s10_write_reg $m $base_addr 0x4 0xff 0x00
s10_write_reg $m $base_addr 0x5 0xff 0x0F
s10_write_reg $m $base_addr 0x6 0xff 0x00
s10_write_reg $m $base_addr 0x7 0xff 0x00

#Register 0x10 : rsfec_top_tx_cfg
s10_write_reg $m $base_addr 0x10 0xff 0x00
s10_write_reg $m $base_addr 0x11 0xff 0x00
s10_write_reg $m $base_addr 0x12 0xff 0x00
s10_write_reg $m $base_addr 0x13 0xff 0x00

#Register 0x14 : rsfec_top_rx_cfg
s10_write_reg $m $base_addr 0x14 0xff 0x11
s10_write_reg $m $base_addr 0x15 0xff 0x11
s10_write_reg $m $base_addr 0x16 0xff 0x00
s10_write_reg $m $base_addr 0x17 0xff 0x00

#Register 0x20 : tx_aib_dsk_conf
s10_write_reg $m $base_addr 0x20 0xff 0x00
s10_write_reg $m $base_addr 0x21 0xff 0x00
s10_write_reg $m $base_addr 0x22 0xff 0x00
s10_write_reg $m $base_addr 0x23 0xff 0x00

#Register 0x30 : rsfec_core_cfg
s10_write_reg $m $base_addr 0x30 0xff 0x80
s10_write_reg $m $base_addr 0x31 0xff 0x00
s10_write_reg $m $base_addr 0x32 0xff 0x00
s10_write_reg $m $base_addr 0x33 0xff 0x00

#Register 0x40 : rsfec_lane_cfg
s10_write_reg $m $base_addr 0x40 0xff 0x08

#Register 0x44 : rsfec_lane_cfg
s10_write_reg $m $base_addr 0x44 0xff 0x08

#Register 0x48 : rsfec_lane_cfg
s10_write_reg $m $base_addr 0x48 0xff 0x08

#Register 0x4C : rsfec_lane_cfg
s10_write_reg $m $base_addr 0x4C 0xff 0x08


close_service master $m
}

###############################################################################
#reconfig_no_fec
###############################################################################

proc reconfig_no_fec {} {

set m [lindex [get_service_paths master] 1]  
open_service master $m


#Reconfigure MAC 

set base_addr 0x00000


#Timer Window for Hi-BER Checks Fields
s10_write_reg_32 $m $base_addr 0x37a 0xffffffff 0x312C7
#EHIP TX MAC Feature Configuration 
s10_write_reg_32 $m $base_addr 0x40b 0xffffffff 0x9FFD8028

#Setup MAC with FEC disabled
#set bit[5] of 0x313
s10_write_reg_32 $m $base_addr 0x313 0xffffffff 0x00000020
#set bit[3] (not documented) and set bit[9] (set use_aligner)
s10_write_reg_32 $m $base_addr 0x30E 0x208 0x00000208

#Fec is already disabled in previous step 


close_service master $m

}

